puts(1,"Copyright (C) Donald Cahela 2003-rkestfun.e\n")
include get.e
include math.e
clear_screen()
--  Functions used to fit parameters in initial value ODE models
--
--  Author:     Donald R. Cahela, Ph.D.
--  Title:      Research Fellow III
--  Address:    Department of Chemical Engineering
--              207 Ross Hall
--              Auburn University, Alabama 36849
--  email:      caheldr@auburn.edu
--  Date:       December 2003

--  Note to anyone wishing to use this program for other models.
--  If you want a model added to this library "rkestfun.e" add the functions 
--  below the ones already in this library and email it to me.  I will 
--  recompile the program with the additional models and post the new version 
--  it to the RDS website: http://www.rapideuphoria.com.
--
--  A program to estimate rate constants in networks of first order reactions
--  from batch type reaction data is also available.  That program does not
--  require any ODE models to be coded.
--  This program estimates the partial derivatives of the system equations
--  with respect to the system variables and parameters numerically.

global function    YALIU_MicrobGrowth_functs(integer ns, atom x, sequence y,
    sequence parmy, sequence cntls)
sequence derv
derv=repeat(0,2)
    derv[1]=parmy[1]*y[1]*(1 - y[1]/parmy[2])
    derv[2]=parmy[3]*y[1]
    return derv
end function

global function    FromentBishoff_Prob117_functs(integer ns, atom x, sequence y,
    sequence parmf, sequence cntls)
sequence derv
derv=repeat(0,8)
    derv[1]=   (parmf[1]+parmf[7]+parmf[8])*(1-y[1])
    derv[2]=   parmf[9]*y[8] + parmf[7]*(1-y[1])
    derv[3]=   parmf[1]*(1-y[1]) - (parmf[2]+parmf[3])*y[3]
    derv[4]=   parmf[2]*y[3] - parmf[4]*y[4]
    derv[5]=   parmf[3]*y[3] - parmf[5]*y[5]
    derv[6]=   parmf[5]*y[5] - parmf[6]*y[6]
    derv[7]=   parmf[4]*y[4] + parmf[6]*y[6]
    derv[8]=   parmf[8]*(1-y[1]) - parmf[9]*y[8]
    return derv
end function

global function    ProlyticDehydrogenationBenzeneHW_functs(integer ns, 
    atom x, sequence y, sequence parmd, sequence cntls)
--  Reference: Seinfeld J. H. and Lapidus L., Mathematical Methods
--  in Chemical Engineering, V3: Process Modeling, Estimation, and
--  Identification, pp. 389-398 (year?)
atom r1, r2, K1, K2
sequence derv
derv=repeat(0,2)
    K1=0.242
    K2=0.428
    r1=parmd[1]*(power(y[1],2)-y[2]*(2-2*y[1]-y[2])/3/K1)
    r2=parmd[2]*(y[1]*y[2]-(1-y[1]-2*y[2])*(2-2*y[1]-y[2])/9/K2)
    derv[1]=-r1-r2
    derv[2]=r1/2-r2
    return derv
end function

global function BARD3d_functs(integer ns, atom x, sequence y, 
    sequence parmb, sequence cntls)
sequence derv
derv=repeat(0,5)
derv[1]=-parmb[1]*y[1]*y[2]+parmb[2]*y[3]
derv[2]=-parmb[1]*y[1]*y[2]+parmb[2]*y[3]
    -parmb[4]*y[2]*y[3]+parmb[5]*y[5]-parmb[6]*y[2]*y[4]
derv[3]=parmb[1]*y[1]*y[2]-(parmb[2]+parmb[3])*y[3]-parmb[4]*y[2]*y[3]
derv[4]=parmb[3]*y[3]+parmb[5]*y[5]-parmb[6]*y[2]*y[4]
derv[5]=parmb[4]*y[2]*y[3]-parmb[5]*y[5]+parmb[6]*y[2]*y[4]
return derv
end function

global function TwoConsFirstOrdRxns_functs(integer ns, atom x, sequence y,
    sequence parmb, sequence cntls)
sequence derv
derv=repeat(0,5)
derv[1]=-parmb[1]*y[1]
derv[2]= parmb[1]*y[1] - parmb[2]*y[2]
derv[3]=                 parmb[2]*y[2]
return derv
end function

global function ThreeCompRevFirstOrdRxns_functs(integer ns, atom x, sequence y,
    sequence parmb, sequence cntls)
sequence derv
derv=repeat(0,3)
derv[1]=-(parmb[1]+parmb[2])*y[1]+parmb[3]*y[2]+parmb[5]*y[3]
derv[2]= parmb[1]*y[1] - (parmb[3]+parmb[4])*y[2] +parmb[6]*y[3]
derv[3]= parmb[2]*y[1] + parmb[4]*y[2] - (parmb[5]+parmb[6])*y[3]
return derv
end function

global function HemicelluloseHydrolysis_functs(integer ns, atom x, sequence y,
    sequence parm, sequence cntls)
sequence derv
atom k1, k2, tref, UGC_calK
--Six parameters
--Two rate constants: k0, E, and acid conc power are parms for each rate constant.
--One control parameter, the reference temperature (150 C for the example).

--Four system equations
--  1: Conc. of first component
--  2: Conc. of second component-measurable component in solution
--  3: Temperature-Centigrade
--  4: Acid concentration-%w/v
    derv=repeat(0,4)
    UGC_calK=1.987
    tref=cntls[1]+273.16  --Reference temperature in Kelvin
    k1=parm[1]*exp(-parm[2]/UGC_calK*(1/(y[3]+273.16)-1/tref))*power(y[4],parm[3])
    k2=parm[4]*exp(-parm[5]/UGC_calK*(1/(y[3]+273.16)-1/tref))*power(y[4],parm[6])
    derv[1]=-k1*y[1]
    derv[2]=k1*y[1]-k2*y[2]
    derv[3]=0
    derv[4]=0
    return derv
end function

global function DiethethylsulfateRxnwSulfuricAcid(integer ns, atom x, sequence y,
 sequence parm, sequence cntls)
sequence derv
--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., John Wiley & Sons,
--1972, pg 88, Problem 3-13.
--Reference to Hellin and Jungers (1957).
--two rate constants
--three system equations
--  1: conc. of diethylsulfate
--  2: conc. of sulfuric acid
--  3: conc. of ethylsulfuric acid
derv=repeat(0,ns)
derv[1]=-parm[1]*y[1]*y[2]+parm[2]*power(y[3],2)
derv[2]=derv[1]
derv[3]=-2*derv[1]
return derv
end function

global function H2plusBr2toHBr(integer ns, atom x, sequence y, 
    sequence parm, sequence cntls)
sequence derv
--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., John Wiley & Sons,
--1972, pg 78, Example 1, Chapter 2.
--Reference to Bodenstein and Lind (1906), etc.
--two rate constants
--three system equations
--  1: conc. of H2
--  2: conc. of Br2
--  3: conc. of HBr
derv=repeat(0,ns)
derv[1]=-parm[1]*y[1]*sqrt(y[2])/(parm[2]+y[3]/y[2])
derv[2]=derv[1]
derv[3]=-2*derv[1]
return derv
end function

global function SucroseHydrolysis(integer ns, atom x, sequence y, 
    sequence parm, sequence cntls)
sequence derv
--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., John Wiley & Sons,
--1972, pg 89, Problem 3-17.
--two parameters
--one system equations
--  1: conc. of sucrose
-- parm1: k3*[enzyme @t=0]
-- parm2: Michaelis constant
derv=repeat(0,ns)
derv[1]=-parm[1]*y[1]/(y[1]+parm[2])
return derv
end function

global function CompartmentalFlowModelK(integer ns, atom x, sequence y,
    sequence parm, sequence cntls)
sequence derv
atom a, b, C0
--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., 
--pg. 303, John Wiley & Sons,1972.
--This model can simulate model K.  
--two parameters
--Model K is a two chamber model where the flow splits between two chambers
--and the output from the two chambers then recombines.
--a is the fraction of the total volume in the first compartment.
--b is the fraction of the flowrate going through the first compartment.
--  Three system equations
--  1:  concentration in fist volume
--  2:  concentration in second volume
--  3:  Overall exit concentration.
--  Control parameters:
--  1:  F/V is average space velocity 1/min
--  2:  Inlet concentration,
--      C0=1 for filling the volume
--      C0=0 for emptying the volume
--      any arbitrary value can be used to represent an arbitrary step change.
--  3:  A flag:
--      0 has no effect-the program simulates a step response
--      y[6]<>0-A square wave input similar to an impulse forcing function to
--      an empty vessel.  C0 = 1 for t=0 to 1, C0 = 0 for t>1
--      Other values for this variable could be used to allow specific
--      forcing functions to be programmed.
--  The parameters are a and b.
--  a:  Fraction of total volume in the first compartment.
--  b:  Fraction of the flowrate going through the first compartment.
derv=repeat(0,ns)
--  a and b are mapped to constrain values between 0 and 1.
a=exp(parm[1])/(exp(parm[1])+exp(-parm[1]))
b=exp(parm[2])/(exp(parm[2])+exp(-parm[2]))
if cntls[3]=0 then
    C0=cntls[2]
else 
--if cntls[3]!=0 then a square forcing function is used, y=1 from t=0 to t=1, y=0 for t>1.
    if x<1 then
	C0=1
    else
	if x=1 then
	    C0=1/2
	else
	    C0=0 
	end if
    end if
end if
derv[1]=cntls[1]*b/a*(C0-y[1])
derv[2]=cntls[1]*(1-b)/(1-a)*(C0-y[2])
derv[3]=b*derv[1]+(1-b)*derv[2]
return derv
end function

global function CompartmentalFlowModelLN(integer ns, atom x, sequence y, 
    sequence parm, sequence cntls)
sequence derv
atom a, b, C0
--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., 
--pg. 303, John Wiley & Sons,1972.
--This model can simulate models L and N.  L is a step input and N is an impulse input.
--two parameters
--Model L and N are two chamber models
--Parm[1] is the fraction of the total volume in the first compartment.
--Parm[2] is the ratio of the exchange flowrate to the throughput.
--Two system equations
--  1: concentration in main volume
--  2: concentration in dead volume
--  Three control parameters:
--  1: F/V is average space velocity 1/min
--  2: Inlet concentration,
--      C0=C0 for filling the volume
--      C0=0 for emptying the volume
--      any arbitrary value can be used to represent an arbitrary step change.
--  3:  A flag:
--      0 has no effect-the program simulates a step response
--      y[5]>0-A square wave input similar to an impulse forcing function to an
--          empty vessel.  C0 = 1 for t=0 to 1, C0 = 0 for t>1
--      Other values for this variable could be used to allow specific
--      forcing functions to be programed.
--  The parameters are a and b.
--  a is the fraction of the vessel volume in the first chamber.
--  b is the ratio of the exchange flowrate to the throughput.
--  parm2:  Ratio of exchange flowrate to F
derv=repeat(0,ns)
--  a is mapped to constrain it between 0 and 1.
a=exp(parm[1])/(exp(parm[1])+exp(-parm[1]))
b=parm[2]
if cntls[2]=0 then
    C0=1
else 
--if cntls[2]!=0 then a square wave forcing function is used.
    if x<1 then
	C0=1
    else
	if x=1 then
	    C0=1/2
	else
	    C0=0 
	end if
    end if
end if
derv[1]=cntls[1]*(C0-(1+b)*y[1]+b*y[2])/a
derv[2]=cntls[1]*b/(1-a)*(y[1]-y[2])
--derv[1]=FdivV/a*(1-(1+b)*y[1]-b*y[2])
--derv[2]=FdivV/(1-a)*b*(y[1]-y[2])
return derv
end function

global function CompartmentalFlowModelM(integer ns, atom x, sequence y,
    sequence parm, sequence cntls)
sequence derv
atom a, b, C0

--Example from Octave Levenspiel, "Chemical Reaction Engineering", 2nd ed., John Wiley & Sons,
--1972.
--two parameters
--Parm[1] is the fraction of the total volume in the first chamber.
--Parm[2] is the ratio of the exchange flowrate to the throughput.
--Five system equations
--  1: concentration in main volume
--  2: concentration in dead volume
--Three control parameters:
--  1: F/V is average space velocity 1/min
--  2: Inlet concentration,
--      C0=C0 for filling the volume
--      C0=0 for emptying the volume
--      any arbitrary value can be used to represent an arbitrary step change.
--  3 is a flag:
--      0 has no effect-the program simulates a step response
--      1-A square wave input similar to an impulse forcing function to an
--          empty vessel.  C0 = 1 for t=0 to 1, C0 = 0 for t>1
--      Other values for this variable could be used to allow specific
--      forcing functions to be programed.
--  a: fraction of total volume in main compartment
--  b: Ratio of circulation rate to total flowrate.
-- parm2: ratio of exchange flowrate to F
    derv=repeat(0,ns)
--a is mapped to constrain it between 0 and 1.
--a is the fraction of the vessel volume in the first chamber.
--b is the ratio of the exchange flowrate to the throughput.
    a=exp(parm[1])/(exp(parm[1])+exp(-parm[1]))
    b=parm[2]
    if cntls[3]=0 then
	C0=cntls[2] 
    else 
    --if cntls[3]!=0 then a square wave forcing function is used.
	if x<1 then
	    C0=1
	else
	    if x=1 then
		C0=1/2
	    else
		C0=0 
	    end if
	end if
    end if
    derv[1]=cntls[1]/a*(C0-(1+b)*y[1]+b*y[2])
    derv[2]=cntls[1]/(1-a)*(1+b)*(y[1]-y[2])
return derv
end function

global function    FromentBishoff_Prob115_functs(integer ns, atom x, sequence y,
    sequence parmf, sequence cntls)
atom Kp
sequence derv
derv=repeat(0,5)
--  1:  Total pressure
--  2:  Partial pressure of N2O5
--  3:  Partial pressure of NO2
--  4:  Partial pressure of O2
--  5:  Partial pressure of N2O4
--Problem 1.15 is a modification of problem 1.14.  Dimerization of the product
--is added to the model.  The solution to problem 1.14 
--is ptot=pN2O5i*(5/2-3/2*exp(-2*k*t).
--Estimation of k from data point at 40 min is 0.003 min^-1.
    Kp=0.0038607    --mmHg^-1
    derv[2]=-2*parmf[1]*y[2]
    derv[3]= 4*parmf[1]*y[2]/(1+4*Kp*y[3])
    derv[4]= parmf[1]*y[2]
    derv[5]= 2*Kp*y[3]*derv[3]
    derv[1]=  derv[2]+derv[3]+derv[4]+derv[5]
    return derv
end function

global function TotalPressDecomposition(integer ns, atom x, sequence y,
    sequence parms, sequence cntls)
--Inital pressure, mole of products per mole of reactants, and 
--order of reaction are input in cntls sequence.
sequence derv
    derv=repeat(0,ns)
    derv[1]=parms[1]*power(cntls[2]*cntls[1]-y[1],cntls[3])
    return derv
end function

include LUdcmp.e

--with trace
--trace(1)

--Auxillary function for Gary_Blau-DOW problem.
function conv_y8y9y10(atom qplus, atom y1, atom y3, atom y5, atom y6, 
	atom K1, atom K2, atom K3)
--Solve algebraic relations for ionic equilibria
atom err, y7, y8, y9, y10
integer count, flag
sequence a, g, dely, phi
    a=repeat(repeat(0,3),3)
    g=repeat(0,3)
    phi=repeat(0,3)     --exp(phi(1))=y8, exp(phi(2))=y9, exp(phi(3))=y10
    dely=g
    err=1
    count=0
--Initial values of search parameters   
    phi[1]=log(1e-8)
    phi[2]=phi[1]
    phi[3]=phi[1]
    
    while err>1e-18 and count<1000 do
	count+=1
--Direct iteration solution method
	--y8b=K2*y1/(K2+y7)
	--y9b=K3*y3/(K3+y7)
	--y10b=K1*y5/(K1+y7)
--Quadratic equation applied to each algebraic equation
	--b=K2+y6+y7+y9+y10
	--y8b=(sqrt(power(b,2)+4*K2*y1)-b)/2
	--b=K3+y6+y7+y8+y10
	--y9b=(sqrt(power(b,2)+4*K3*y3)-b)/2
	--b=K1+y6+y7+y8+y9
	--y10b=(sqrt(power(b,2)+4*K1*y5)-b)/2
	--if y8!=0 then
	    --err=power(dely[1]/y8,2)
	--else
	    --err=power(dely[1],2)
	--end if
	--if y9!=0 then
	    --err+=power(dely[2]/y9,2)
	--else
	    --err+=power(dely[2],2)
	--end if
	--if y10!=0 then
	    --err+=power(dely[3]/y10,2)
	--else
	    --err+=power(dely[3],2)
	--end if
	--err=sqrt(err)/3
--Newton-Ralphson search applied to three algebraic equations resulting from
--the equilibrium relationships.
--Search in linear space.
	--a[1][1]=K2+2*y8+(y6+y9+y10-qplus)
	--a[1][2]=y8
	--a[1][3]=y8
	
	--a[2][1]=y9
	--a[2][2]=K3+2*y9+(y6+y8+y10-qplus)
	--a[2][3]=y9
	
	--a[3][1]=y10
	--a[3][2]=y10
	--a[3][3]=K1+2*y10+(y6+y8+y9-qplus)
	
	--y7=y6+y8+y9+y10-qplus
	
	--g[1]=(K2+y7)*y8-K2*y1
	--g[2]=(K3+y7)*y9-K3*y3
	--g[3]=(K1+y7)*y10-K1*y5

	--dely=Axb_Solve(3, a, -g)
	
	--y8+=dely[1]
	--y9+=dely[2]
	--y10+=dely[3]
	
	--if y8<0 then
	    --y8=0
	--end if
	--if y9<0 then
	    --y9=0
	--end if
	--if y10<0 then
	    --y10=0
	--end if
--Newton-Ralphson search applied to three algebraic equations resulting from
--the equilibrium relationships.
--Search in exponential space.
	y8=exp(phi[1])
	y9=exp(phi[2])
	y10=exp(phi[3])
	y7=y6+y8+y9+y10-qplus
	
	a[1][1]=(K2+y7+y8)*y8
	a[1][2]=y8*y9
	a[1][3]=y8*y10
	
	a[2][1]=y9*y8
	a[2][2]=(K3+y9+y7)*y9
	a[2][3]=y9*y10
	
	a[3][1]=y10*y8
	a[3][2]=y10*y9
	a[3][3]=(K1+y7+y10)*y10
	
	y7=y6+y8+y9+y10-qplus
	
	g[1]=-((K2+y7)*y8-K2*y1)
	g[2]=-((K3+y7)*y9-K3*y3)
	g[3]=-((K1+y7)*y10-K1*y5)

	dely=Axb_Solve(3, a, g)
	
	flag=0
	for i=1 to 3 do
	    if abs(dely[i]>200) then
		flag=1
	    end if
	end for
	
	while flag=1 do
	    flag=0
	    for i=1 to 3 do
		dely[i]/=2
		if abs(dely[i]>200) then
		    flag=1
		end if
	    end for
	end while
	
	for i=1 to 3 do
	    phi[i]+=dely[i]
	end for
	
	err=0
	for i=1 to 3 do
	    err+=power(g[i],2)
	end for
	err=sqrt(err/3)
	printf(1,"count=%d, y7=%g, y8=%g, y9=%g, err=%g\n",{count,y7,y8,y9,err})
    end while

    y8=exp(phi[1])
    y9=exp(phi[2])
    y10=exp(phi[3])
    y7=y6+y8+y9+y10-qplus
    if count>=100 then
	printf(1,"Iteration count in aux func for Gary Blau-DOW problem=%d, err=%g\n",{count,err})
    end if
    return {y7,y8,y9,y10,err}
end function

--test algebraic solution
--puts(1,"Test algebraic solution of auxilary equation in Gary Blau-DOW problem\n")
--atom y1, y3, y5, y6, qplus, K1, K2, K3--, y7, y8, y9, y10
--sequence z
--z=repeat(0,4)
--y1=1.6
--y3=0.01
--y5=0
--y6=0.0131
--qplus=0.0131
--K1=1e-17
--K2=1e-11
--K3=6e-18
--z=conv_y8y9y10(qplus, y1, y3, y5, y6, K1, K2, K3)
--printf(1,"y7=%g, y8=%g, y9=%g, y10=%g, err=%g\n",z)

global function Gary_Blau_DOW_Test_Prob(integer ns, atom x, sequence y,
    sequence parms, sequence cntls)
--Reference:L. T. Biegler and J. J. Damiano and G. E. Blau,
--          "Nonlinear Parameter estimation: a Case Study Comparison,"
--          AIChE Journal (Vol. 32, No. 1), pg. 29-45, (1986). 
--  ns=(6)7, np=9, y[7]=Temperature-C
sequence derv, z
atom y7, y8, y9, y10, qplus, k1, k2, km1, T0
atom UGC_JmolK
UGC_JmolK=8.314     --Joule/gmol-K
    derv=repeat(0,ns)
    z=repeat(0,5)
    qplus=cntls[1]
    T0=cntls[2]
    z=conv_y8y9y10(qplus, y[1], y[3], y[5], y[6], parms[7], parms[8], parms[9])
    y7=z[1]
    y8=z[2]
    y9=z[3]
    y10=z[4]
--Temperature is y[7] with a zero derivative, ns=7.
    --k1=parms[1]*exp(-parms[2]/UGC_JmolK*(1/(y[7]+273.16)-1/(T0+273.16)))
    --k2=parms[3]*exp(-parms[4]/UGC_JmolK*(1/(y[7]+273.16)-1/(T0+273.16)))
    --km1=parms[5]*exp(-parms[6]/UGC_JmolK*(1/y[7]-1/T0))
    k1=parms[1]*exp(-parms[2]/UGC_JmolK/(y[7]+273.16))
    k2=parms[3]*exp(-parms[4]/UGC_JmolK/(y[7]+273.16))
    km1=parms[5]*exp(-parms[6]/UGC_JmolK/(y[7]-1/T0))
    derv[1]=-k2*y8*y[2]
    derv[2]=-k1*y[6]*y[2]+km1*y10-k2*y8*y[2]
    derv[3]=-k2*y8*y[2]+k1*y[6]*y[4]-km1*y9/2
    derv[4]=-k1*y[6]*y[4]+km1*y9/2
    derv[5]=-k1*y[6]*y[2]-km1*y10
    derv[6]=-k1*y[6]*(y[2]+y[4])+km1*(y10+y9/2)
    return derv
end function


--Estimate effective thermal conductivity and wall heat transfer coefficient
--from heat up of a packed tube.  Internal diameter, tube thickness, 
--thermal conductivity of tube material, and total volumetric heat capacity are
--input as a contol parameters.
global function inflongtubeheatuphwcntls(integer ns, atom x, sequence y, 
    sequence parm, sequence cntls)
sequence derv
atom eta, deta, detasq, kRh, R, W, kw, densCp, km, kmdkwln1pTdR
--Heat up of an infinitely long packed tube using dimensionless temperature
--  and dimensionless radius: Y=(T-T0)/(Tinf-T0), eta=r/R
--  one parameter: alpha = k/(dens*Cp)/R^2 
--  one control parmeter = k/(h*R)
--  delta eta=1/ns, r(ns)=R*(1-deta)
--  pT/pt=k/(dens*Cp)*(1/r*pT/pr+p2T/pr2)
--  pY/pt=alpha*(1/eta*p/p-eta(eta*pY/p-eta)
--  Material has thermal conductivity-k (W/m-K), Cp-J/kg-K, density-kg/m3
--  Wall heat transfer coefficient-h (W/m2-K)
--  r[i]=delr*(i-1), values at 0<r<R-delr are integrated over time.
    kRh=parm[2]     --k/R*hwall
    R=cntls[1]      --Tube I.D., meter  
    W=cntls[2]      --Wall thickness, meter
    kw=cntls[3]     --Wall material thermal conductivity, W/m-K
    densCp=cntls[4] --Volumetric heat capacity of packing, J/m3-K
    km=parm[1]*densCp*power(R,2)    --W/m-K
    kmdkwln1pTdR= km/kw*log(1+W/R)
    
--  Usually there are n divisions and n+1 points.  
--  In this case n-1 divisions and n points are used in the simulation.
    deta=1/(ns-1)
    detasq=power(deta,2)
    derv=repeat(0,ns)
    derv[1]=parm[1]/detasq*2*(y[2]-y[1])
    for i=2 to ns-1 do
	eta=(i-1)*deta
	--derv[i]=parm[1] * ( 1/eta * (y[i+1]-y[i-1])/deta
	    --+ (y[i-1]-2*y[i]+y[i+1])/detasq)
	derv[i]=parm[1]/detasq*((2*i-3)*y[i-1] - 4*(i-1)*y[i] + (2*i-1)*y[i+1])
	    /(2*(i-1))
    end for

--B.C. at wall is incorporated into the derv at one deta inside of wall.
--Heat transfer through pipe wall is incorporated into the y[ns+1] estimation
-- using a pseudo-steady state assumption.
--y[ns+1]=(1+(k/hR+km/kw*ln(1+T/R))*(4*y[ns]-y[ns-1])/(2*deta))/(1+3*(k/hR+km/kw*ln(1+T/R))/(2*deta))
    derv[ns]=parm[1]/detasq*(
    (2*ns-3)*y[ns-1] - 4*(ns-1)*y[ns] 
+ (2*ns-1) * (1+(kRh+kmdkwln1pTdR)*(4*y[ns]-y[ns-1])/(2*deta))
	/(1+3*(kRh+kmdkwln1pTdR)/(2*deta)) ) /(2*(ns-1))
    return derv
end function


--Estimate effective thermal conductivity and wall heat transfer coefficient
--from heat up of a packed tube.  Estimate alpha/R^2 and k/h*R.
global function heatuptubewithflow(integer ns, atom x, sequence y, 
    sequence parm, sequence cntls)
sequence derv
atom eta, deta, detasq, kRh, R, W, kw, densCps, densCpf, km, kmdkwln1pTdR
atom L, vz, vzL, RL2, dx, dx2, ynrp1
integer nr, nz, k, krp1, krm1
--  Simulation of heat up of a packed tube using dimensionless temperature
--  Both ends of the tube are insulated, dT/dz=0 at x=0 and x=L.
--  and dimensionless radius: Y=(T-T0)/(Tinf-T0), eta=r/R
--  one parameter: alpha = k/(dens*Cp)/R^2 
--  one control parmeter = k/(h*R)
--  delta eta=1/ns, r(ns)=R*(1-deta)
--  pT/pt=k/(dens*Cp)*(1/r*pT/pr+p2T/pr2)
--  pY/pt=alpha*(1/eta*p/p-eta(eta*pY/p-eta)
--  Material has thermal conductivity-k (W/m-K), Cp-J/kg-K, density-kg/m3
--  Wall heat transfer coefficient-h (W/m2-K)
--  r[i]=delr*(i-1), values at 0<r<R-delr are integrated over time.
    kRh=parm[2]
    
    R=cntls[1]      --meter
    W=cntls[2]      --Tube thickness-meter
    kw=cntls[3]     --W/m-K
    densCps=cntls[4] --J/m3-K
    densCpf=cntls[5]
    L=cntls[6]      --meter
    vz=cntls[7]     --m/s
    nr=cntls[8]
    nz=cntls[9]
    
    km=parm[1]*densCps*power(R,2)    --W/m-K
    kmdkwln1pTdR= km/kw*log(1+W/R)
    vzL=vz/L*densCpf/densCps
    RL2=power(R/L,2)
    
    deta=1/nr
    detasq=power(deta,2)
    dx=1/nz
    dx2=power(dx,2)
--  ns=nr*nz
    derv=repeat(0,ns)

--The boundary at the entrance of the reactor, T[i,0]=T[i,2]
--Take the second derv of temp wrsz=0 at z=0 or x=0. y[i,0]=y[i,2]
	k=1
	krp1=nz+1
	---vzL*(-3*y[k]+4*y[k+1]-y[k+2])/(2*dx)
	--derv[1]= parm[1]/detasq*(2*(y[krp1]-y[k])
	    --+ 2*RL2*(y[k+1]-y[k])/dx2)
	derv[1]=0
	    
	for i=2 to nr-1 do
	    --eta=(i-1)*deta
	    k=(i-1)*nz+1
	    --krm1=(i-2)*nz+1
	    --krp1=i*nz+1
	    ---vzL*(y[k+1]-y[k])/dx
	    --derv[k] = ((2*i-3)*y[krm1] - 4*(i-1)*y[k] + (2*i-1)*y[krp1])/(2*(i-1))
	    --derv[k]+= 2*RL2*(y[k+1]-y[k])/dx2 
	    --derv[k]*= parm[1]/detasq
	    derv[k]=0
	end for
--B.C. at wall is incorporated into the derv at one deta inside of wall.
--Heat transfer through pipe wall is incorporated into the y[ns+1] estimation
-- using a pseudo-steady state assumption.
--y[ns+1]=(1+(k/hR+km/kw*ln(1+T/R))*(4*y[ns]-y[ns-1])/(2*deta))/(1+3*(k/hR+km/kw*ln(1+T/R))/(2*deta))
--  i=nr, j=1,2,..,nz-1     
	k=(nr-1)*nz+1
	--ynrp1= (1+(kRh+kmdkwln1pTdR)*(4*y[k]-y[k-1])/(2*deta))/(1+3*(kRh+kmdkwln1pTdR)/(2*deta))
	--derv[k]= ((2*nr-3)*y[k-1] - 4*(nr-1)*y[k] + (2*nr-1)* ynrp1)/(2*(nr-1))/detasq
	--derv[k]+= 2*RL2*(y[k+1]-y[k])/dx2
	--derv[k]*= parm[1]
	derv[k]=0
	    

--  Points from j=2 to j=nz-1 or z=dz to z=L-dz 
    for j=2 to nz-1 do
	k=j
	krp1=nz+j
	derv[k]=2*(y[krp1]-y[k])/detasq+RL2*(y[k-1]-2*y[k]+y[k+1])/dx2
	derv[k]*=parm[1]
	derv[k]-=vzL*(y[k+1]-y[k-1])/(2*dx)
	for i=2 to nr-1 do
	    eta=(i-1)*deta
	    k=(i-1)*nz+j
	    krm1=(i-2)*nz+j
	    krp1=i*nz+j
	    derv[k] = ((2*i-3)*y[krm1] - 4*(i-1)*y[k] + (2*i-1)*y[krp1])/(2*(i-1))/detasq
	    derv[k]+= RL2*(y[k-1]-2*y[k]+y[k+1])/dx2
	    derv[k] *= parm[1]
	    derv[k]-=vzL*(y[k+1]-y[k-1])/(2*dx) 
	end for
--B.C. at wall is incorporated into the derv at one deta inside of wall.
--Heat transfer through pipe wall is incorporated into the y[ns+1] estimation
-- using a pseudo-steady state assumption.
--y[ns+1]=(1+(k/hR+km/kw*ln(1+T/R))*(4*y[ns]-y[ns-1])/(2*deta))/(1+3*(k/hR+km/kw*ln(1+T/R))/(2*deta))
	k=nr*nz
	krm1=(nr-1)*nz
	ynrp1=(1+(kRh+kmdkwln1pTdR)*(4*y[k]-y[krm1])/(2*deta))
		/(1+3*(kRh+kmdkwln1pTdR)/(2*deta))
	derv[k]=((2*nr-3)*y[krm1] - 4*(nr-1)*y[k]+ (2*nr-1) * ynrp1)
	    /detasq/(2*(nr-1)) 
	derv[k]+= 2*RL2*(y[k-1]-y[k])/dx2
	derv[k]*=parm[1]
	derv[k]-=vzL*(y[k-2]-4*y[k-1]+3*y[k])/dx
    end for

--The boundary at the exit of the reactor, T[nz+1]=T[nz-1]
--Take the second derv of temp wrsz=0 at z=L or x=1. y[nz+1]=y[nz-1]
    k=nz
    krp1=nz+nz
    ---vzL*(y[k-2] -4*y[k-1] + 3*y[k])/(2*dx)
    derv[k]= parm[1]/detasq*( (y[k+1]-y[k])/dx + 2*RL2*(y[k-1]-y[k])/dx2)
    for i=2 to nr-1 do
	eta=(i-1)*deta
	k=i*nz
	krm1=(i-1)*nz
	krp1=(i+1)*nz
	---vlL*(y[kzp1]-y[kzm1])/(2*dx)
	derv[k]=((2*i-3)*y[krm1] - 4*(i-1)*y[k] + (2*i-1)*y[krp1])/(2*(i-1))/detasq
	derv[k]+=2*RL2*(y[k-1]-y[k])/dx2
	derv[k]*= parm[1]
    end for
--B.C. at wall is incorporated into the derv at one deta inside of wall.
--Heat transfer through pipe wall is incorporated into the y[ns+1] estimation
-- using a pseudo-steady state assumption.
--y[ns+1]=(1+(k/hR+km/kw*ln(1+T/R))*(4*y[ns]-y[ns-1])/(2*deta))/(1+3*(k/hR+km/kw*ln(1+T/R))/(2*deta))
    k=nr*nz
    krm1=(nr-1)*nz
--  -vlL*(y[k]-y[k-1])/dx
    ynrp1=(1+(kRh+kmdkwln1pTdR)*(4*y[k]-y[krm1])/(2*deta))/(1+3*(kRh+kmdkwln1pTdR)/(2*deta))
    derv[k]=((2*nr-3)*y[krm1] - 4*(nr-1)*y[k] + (2*nr-1) * ynrp1)/(2*(nr-1)) /detasq
	+ 2*RL2*(y[k-1]-y[k])/dx2
    derv[k]*=parm[1]
    
    return derv
end function

global function YYLeeHemicelluloseHydrolysisModel(integer ns, atom t, sequence y,
    sequence parms, sequence cntls)
sequence derv
atom kf, ks, xf0, xs0, xf, xs
    derv=repeat(0,ns)
    xf0=cntls[1]
    xs0=cntls[2]
    kf=cntls[3]
    ks=cntls[4]
    xf=xf0*exp(-kf*t)
    xs=xs0*exp(-ks*t)
--  Components are: 1-Oligimers, 2-xylose, 3-Lignin(solid), 4-D
--      5-LLC1, 6-LLC2, 7-Lignin(soluble)
    derv[1]= kf*xf + ks*xs - parms[1]*y[1] - parms[3]*y[1]*y[3]
    derv[2]= parms[1]*y[1] - parms[2]*y[2] - parms[4]*y[2]*y[3]
    derv[3]= parms[5]*y[7] - parms[3]*y[1]*y[3] - parms[4]*y[2]*y[3]
    derv[4]= parms[2]*y[2]
    derv[5]= parms[3]*y[1]*y[3]
    derv[6]= parms[4]*y[2]*y[3]
    derv[7]=-parms[5]*y[7]
    return derv
end function

--*************************************************************************
global function numerical_pdYdtpY(integer ns, integer np, atom xn, sequence yn, 
    sequence parmn, sequence cntls, integer sysderv_id)
--Numerical computation of partial derivates of system equations.
atom dely
sequence dervs0, dervs1, yn1, a
    dervs0=repeat(0,ns)
    dervs1=repeat(0,ns)
    yn1=repeat(0,ns)
    a=repeat(repeat(0,ns),ns)
    dervs0=call_func(sysderv_id,{ns,xn,yn,parmn,cntls})
    for j=1 to ns  do
	if abs(yn[j])>=1e-100 then    dely=0.001*yn[j]    else    dely=1e-10   end if
	yn[j]+=dely
	dervs1=call_func(sysderv_id,{ns,xn,yn,parmn,cntls})
	yn[j]-=dely
	for i=1 to ns  do
	    a[i][j]=(dervs1[i]-dervs0[i])/dely
	end for
    end for
    return a
end function

global function numerical_pdYdtpk(integer ns, integer np, atom xn, sequence yn, 
    sequence parmn, sequence cntls, integer sysderv_id)
--Numerical computation of partial derivates of system equations.
atom delp
sequence dervs0, dervs1, yn1, b
    dervs0=repeat(0,ns)
    dervs1=repeat(0,ns)
    yn1=repeat(0,ns)
    b=repeat(repeat(0,np),ns)
    dervs0=call_func(sysderv_id,{ns,xn,yn,parmn,cntls})
    for k=1 to np  do
	if parmn[k]!=0 then delp=0.001*parmn[k] else    delp=0.000001   end if
	parmn[k]+=delp
	dervs1=call_func(sysderv_id,{ns,xn,yn,parmn,cntls})
	parmn[k]-=delp
	for i=1 to ns  do
	    b[i][k]=(dervs1[i]-dervs0[i])/delp
	end for
    end for
    return b
end function


function first_ten_choices(integer maxmodels)
integer mcno    
    printf(1,"Reaction model library has %d choices:\n",maxmodels)
    puts(1,"First ten ODE model choices\n")
    puts(1,"0: ODE Model is not in this set\n")
    puts(1,"1: Microbial Growth Model\n")
    puts(1,"2: Problem 1.17 from Froment and Bishoff, 1979, ns=8, np=9\n")
    puts(1,"3: Pyrolytic dehydrogenation of Benzene\n")
    puts(1,"4: Y. BARD's Problem 3d,f, ns=5,np=6\n")
    puts(1,"5: A->B->C, Two consecutive first order reactions ns=3,np=2\n")
    puts(1,"6: Three component reversible network, ns=3,np=6\n")
    puts(1,"7: Hemicellulose Hydrolysis:Two cons 1st Ord Rxns,\n")
    puts(1,"      Arrhenius parms and acid power, ns=4,np=6\n")
    puts(1,"8: A+B=2C ,ns=3,np=2\n")
    puts(1,"9: H2 + Br2 = HBr,ns=3,np=2\n")
    puts(1,"10: SucroseHydrolysis,ns=1,np=2\n")
    mcno=prompt_number("Enter Number of ODE Model to Fit, 0-10\n",{0,10})
    return mcno
end function

function second_ten_choices(integer maxmodels)
integer mcno    
    printf(1,"Reaction model library has %d choices:\n",maxmodels)
    printf(1,"Model choices 11 to %d\n",maxmodels)
    puts(1,"0 : ODE Model is not in this set\n")
    puts(1,"11: CompartmentalFlowModelK, ns=3, np=2, nc=3\n")
    puts(1,"12: CompartmentalFlowModelM, ns=2, np=2, nc=3\n")
    puts(1,"13: CompartmentalFlowModelN, ns=2, np=2, nc=3\n")
    puts(1,"Compartmental flow models can be used to fit three types of data:\n")
    puts(1,"1) filling, 2) emptying, 3) square wave input\n")
    puts(1,"     -similar to impulse forcing function\n")
    puts(1,"14: Problem 1.15 from Froment and Bishoff, 1979, A total pressure measurement,\n")
    puts(1,"     4 components, 3 rate constants\n")
    puts(1,"15: Total pressure method for decomposition: A->nB, dP/dt=k*(n*P0-P)^alpha\n")
    puts(1,"16: Gary Blau-DOW Test Problem\n")
    puts(1,"17: Heatup of infinitely long packed tube: R, T, kw, densCp input as control\n")
    puts(1,"18: Heatup of packed tube with flow\n")
    puts(1,"19: Hemicellulose Hydrolysis Model-Nonlinear \n")
    mcno=prompt_number("Enter Number of ODE Model to Fit, 0-19\n",{0,maxmodels})
    return mcno
end function

--**********************Choose model to fit data to*********************
global integer pdYdtpY_id, pdYdtpk_id
pdYdtpY_id=routine_id("numerical_pdYdtpY")
pdYdtpk_id=routine_id("numerical_pdYdtpk")

global function choose_ODE_model()

integer modelchoice, maxmodels, sysdervs_id
    maxmodels=19
    modelchoice=0
    while ((modelchoice<1) or (modelchoice>maxmodels)) do
	modelchoice=first_ten_choices(maxmodels)
	
	clear_screen()
	if modelchoice=0 then
	    modelchoice=second_ten_choices(maxmodels)
	    clear_screen()
	end if
    
	if modelchoice=1 then
	    sysdervs_id=routine_id("YALIU_MicrobGrowth_functs")
	end if
    
	if modelchoice=2 then
	    sysdervs_id=routine_id("FromentBishoff_Prob117_functs")
	end if
    
	if modelchoice=3 then
	    sysdervs_id=routine_id("ProlyticDehydrogenationBenzeneHW_functs")
	end if

	if modelchoice=4 then
	    sysdervs_id=routine_id("BARD3d_functs")
	end if
	
	if modelchoice=5 then
	    sysdervs_id=routine_id("TwoConsFirstOrdRxns_functs")
	end if
	
	if modelchoice=6 then
	    sysdervs_id=routine_id("ThreeCompRevFirstOrdRxns_functs")
	end if
    
	if modelchoice=7 then
	    sysdervs_id=routine_id("HemicelluloseHydrolysis_functs")
	end if
    
	if modelchoice=8 then
	    sysdervs_id=routine_id("DiethethylsulfateRxnwSulfuricAcid")
	end if

	if modelchoice=9 then
	    sysdervs_id=routine_id("H2plusBr2toHBr")
	end if
    
	if modelchoice=10 then
	    sysdervs_id=routine_id("SucroseHydrolysis")
	end if

	if modelchoice=11 then
	    sysdervs_id=routine_id("CompartmentalFlowModelK")
	end if

	if modelchoice=12 then
	    sysdervs_id=routine_id("CompartmentalFlowModelLN")
	end if

	if modelchoice=13 then
	    sysdervs_id=routine_id("CompartmentalFlowModelM")
	end if

	if modelchoice=14 then
	    sysdervs_id=routine_id("FromentBishoff_Prob115_functs")
	end if

	if modelchoice=15 then
	    sysdervs_id=routine_id("TotalPressDecomposition")
	end if
	
	if modelchoice=16 then
	    sysdervs_id=routine_id("Gary_Blau_DOW_Test_Prob")
	end if
	
	if modelchoice=17 then
	    sysdervs_id=routine_id("inflongtubeheatuphwcntls")
	end if
	
	if modelchoice=18 then
	    sysdervs_id=routine_id("heatuptubewithflow")
	end if
	
	if modelchoice=19 then
	    sysdervs_id=routine_id("YYLeeHemicelluloseHydrolysisModel")
	end if

    end while
    return sysdervs_id
end function

puts(1,"\nrkestfun.e copyright Donald R. Cahela 2003\n")
